@@ -8,7 +8,9 @@ module Text.XmlHtml.HTML.Meta
   , isRawText
   , endOmittableLast
   , endOmittableNext
+  , explicitAttributes
   , predefinedRefs
+  , reversePredefinedRefs
   ) where
 
 #if !MIN_VERSION_base(4,8,0)
@@ -112,6 +114,16 @@ endOmittableNext = M.fromList [
     ("tr",       S.fromList ["tr"])
     ]
 
+------------------------------------------------------------------------------
+-- | Tags and attributes which should always be rendered with an explicit
+-- value, even when the value is empty.  This is required by some web browsers
+-- for tags that are typically non-empty.
+{-# NOINLINE explicitAttributes #-}
+explicitAttributes :: HashMap Text (HashSet Text)
+explicitAttributes = M.fromList [
+    ("a", S.fromList ["href"])
+    ]
+
 ------------------------------------------------------------------------------
 -- | Predefined character entity references as defined by the HTML5 spec.
 {-# NOINLINE predefinedRefs #-}
@@ -2534,3 +2546,10 @@ reftab58 =
     ("zwj", "\x0200D"),
     ("zwnj", "\x0200C") ]
 
+
+------------------------------------------------------------------------------
+-- Reverse lookup of Html entities
+reversePredefinedRefs :: HashMap Text Text
+reversePredefinedRefs = M.fromList . map (\(x,y) -> (y,x)) $
+                        M.toList predefinedRefs
+
